<?php

namespace Drupal\webform_post_remote_queue\Plugin\WebformHandler;

use Drupal\webform\Plugin\WebformHandler\RemotePostWebformHandler;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Webform submission remote post handler.
 *
 * @WebformHandler(
 *   id = "webform_post_remote_queue",
 *   label = @Translation("Remote post (Queued)"),
 *   category = @Translation("External"),
 *   description = @Translation("Posts webform submissions to a URL leveraging a Queue."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_REQUIRED,
 *   tokens = TRUE,
 * )
 */
class RemotePostQueueWebformHandler extends RemotePostWebformHandler {

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    $state = $webform_submission->getWebform()->getSetting('results_disabled') ? WebformSubmissionInterface::STATE_COMPLETED : $webform_submission->getState();
    $this->remotePostQueue($state, $webform_submission);
  }

  /**
   * Queue a remote post.
   *
   * @param string $state
   *   The state of the webform submission.
   *   Either STATE_NEW, STATE_DRAFT_CREATED, STATE_DRAFT_UPDATED,
   *   STATE_COMPLETED, STATE_UPDATED, or STATE_CONVERTED
   *   depending on the last save operation performed.
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   The webform submission to be posted.
   *
   * @see \Drupal\webform\Plugin\WebformHandler::remotePost
   */
  protected function remotePostQueue($state, WebformSubmissionInterface $webform_submission) {
    /** @var \Drupal\Core\Queue\QueueFactory $queue_factory */
    $queue_factory = \Drupal::service('queue');
    $queue = $queue_factory->get('webform_post_remote_queue');
    $item = [
      'configuration' => $this->configuration,
      'state' => $state,
      'webform_submission_id' => $webform_submission->id(),
      'handler_id' => $this->getHandlerId(),
    ];
    $queue->createItem($item);
  }

}
